describe ActiveSchema::Column do
  include_setup :*

  before do
    model.schema {|t| t.string :name}
  end

  it 'creates columns that does not exists' do
    mock(connection, table_exists?: true)

    expect(connection).
      to_receive(:column_exists?).
      with {|*a| a.include?('name')}.
      and_return(false)

    expect(connection).
      to_receive(:add_column).
      with {|t,c,*| t == model.table_name && c == 'name'}

    model.apply_schema
  end

  it 'does not try to create columns that already exists' do
    mock(connection, table_exists?: true, column_exists?: true)
    spy(connection, :add_column)
    model.apply_schema
    assert(connection).did_not.received(:add_column)
  end

  it 'updates column type' do
    model.apply_schema
    assert(connection.columns(model.table_name).map(&:name)).include?('name')
    model.schema {|t| t.text :name}

    expect(connection).
      to_receive(:change_column).
      with(model.table_name, 'name', :text, {})

    model.apply_schema
  end

  it 'updates column options' do
    model.apply_schema
    column = connection.columns(model.table_name).find {|c| c.name == 'name'}
    assert(column.class.name) =~ /Column/
    is(column.default).nil?

    model.schema {|t| t.string :name, default: 'blah'}

    expect(connection).
      to_receive(:change_column).
      with(model.table_name, 'name', :string, {default: 'blah'})

    model.apply_schema
  end

  it 'updates both type and options' do
    model.apply_schema
    model.schema {|t| t.text :name, limit: 50}

    # called once to update type
    expect(connection).
      to_receive(:change_column).
      with(model.table_name, 'name', :text, {})

    # and once to update options
    expect(connection).
      to_receive(:change_column).
      with(model.table_name, 'name', :text, {limit: 50})

    model.apply_schema
  end
end
